package com.install4j.api.beaninfo;

import java.beans.IntrospectionException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyDescriptor;

/**
 * Property descriptor for use in instances of {@link Install4JBeanInfo}.
 * For file and script properties, derived classes are available.
 * <p>Using this class is not strictly required. In principle, you could also set values for the <tt>ATTRIBUTE_*</tt> constants in the property descriptor
 * of an unrelated property descriptor class.
 */
public class Install4JPropertyDescriptor extends PropertyDescriptor {

    /**
     * @see #setPropertyCategory(String)
     */
    public static final String ATTRIBUTE_PROPERTY_CATEGORY = "propertyCategory";

    /**
     * Default category used by install4j.
     */
    public static final String CATEGORY_CONFIGURATION = "Configuration";

    /**
     * @see #create(String, Class, String, String, String)
     */
    public static final String ATTRIBUTE_CONTEXT = "context";

    /**
     * Special context for properties of type <tt>java.lang.String</tt> that tells the install4j GUI
     * to treat the string value as the id of a launcher. In the property panel, a drop-down list of
     * all defined launchers will be shown.
     */
    public static final String CONTEXT_LAUNCHER_ID = "launcher";

    /**
     * Same as {@link #CONTEXT_LAUNCHER_ID}, but service launchers are not selectable.
     */
    public static final String CONTEXT_NON_SERVICE_LAUNCHER_ID = "nonServiceLauncher";

    /**
     * Same as {@link #CONTEXT_LAUNCHER_ID}, but only service launchers are selectable.
     */
    public static final String CONTEXT_SERVICE_LAUNCHER_ID = "serviceLauncher";

    /**
     * Special context for properties of type <tt>java.lang.String</tt> that tells the install4j GUI
     * to allow the user to enter multi-line strings in a separate dialog.
     */
    public static final String CONTEXT_MULTILINE = "multiline";

    /**
     * Special context for properties of type <tt>java.lang.String</tt> that tells the install4j GUI
     * to allow the user to enter an HTML document in a separate editor with syntax coloring and preview
     * functionality. 
     */
    public static final String CONTEXT_HTML = "html";

    /**
     * Special context for properties of type <tt>java.util.Date</tt> that tells the install4j GUI
     * to offer a time editor besides the default date editor as well.
     */
    public static final String CONTEXT_DATETIME = "dateTime";

    /**
     * @see #setSortKey(Integer)
     */
    public static final String ATTRIBUTE_SORT_KEY = "sortKey";

    /**
     * @see #setPropertyChangeListener(java.beans.PropertyChangeListener)
     */
    public static final String ATTRIBUTE_PROPERTY_CHANGE_LISTENER = "propertyChangeListener";

    /**
     * @see #setParentProperty(String) 
     */
    public static final String ATTRIBUTE_PARENT_PROPERTY = "parentProperty";

    /**
     * @see #setVisibilityDiscriminator(VisibilityDiscriminator)
     */
    public static final String ATTRIBUTE_VISIBILITY_DISCRIMINATOR = "visibilityDiscriminator";


    /**
     * Same as {@link #create(String, Class, String, String, String)} with a <tt>context</tt> of <tt>null</tt>.
     */
    public static Install4JPropertyDescriptor create(String propertyName, Class beanClass, String displayName, String shortDescription) {
        return create(propertyName, beanClass, displayName, shortDescription, null);
    }

    /**
     * Create a property descriptor that can be passed to {@link Install4JBeanInfo#addPropertyDescriptor(Install4JPropertyDescriptor)}.
     * @param propertyName the name of the property
     * @param beanClass the class of the bean that contains th property. <b>Note:</b>This is <b>not</b> the class of the property.
     * @param displayName the display name of the property
     * @param shortDescription a short description of the property
     * @param context the context of the property. The context allows to specify different editors for the same property types.
     * See the <tt>CONTEXT_*</tt> properties for the built-in contexts. This context is also used for enumerated properties
     * in the {@link EnumerationMapper}. For properties of type <tt>String[]</tt>, you can pass an item name as the context.
     * @return the property descriptor
     */
    public static Install4JPropertyDescriptor create(String propertyName, Class beanClass, String displayName, String shortDescription, String context) {

        try {
            Install4JPropertyDescriptor descriptor = createDescriptor(propertyName, beanClass);
            descriptor.setDisplayName(displayName);
            descriptor.setShortDescription(shortDescription);
            descriptor.setContext(context);
            return descriptor;
        } catch (IntrospectionException e) {
            e.printStackTrace();
            return null;
        }
    }

    private static Install4JPropertyDescriptor createDescriptor(String propertyName, Class beanClass) throws IntrospectionException {
        try {
            return new Install4JPropertyDescriptor(propertyName, beanClass);
        } catch (Exception ex) {
        }
        try {
            return new Install4JPropertyDescriptor(propertyName, beanClass, "is" + capitalize(propertyName), null);
        } catch (Exception ex) {
        }
        return new Install4JPropertyDescriptor(propertyName, beanClass, "get" + capitalize(propertyName), null);

    }

    static String capitalize(String name) {
        if (name == null || name.length() == 0) {
            return name;
        }
        return name.substring(0, 1).toUpperCase() + name.substring(1);
    }

    Install4JPropertyDescriptor(String propertyName, Class beanClass) throws IntrospectionException {
        super(propertyName, beanClass);
    }

    Install4JPropertyDescriptor(String propertyName, Class beanClass, String readMethod, String writeMethod) throws IntrospectionException {
        super(propertyName, beanClass, readMethod, writeMethod);
    }

    /**
     * Set the context for the property.
     * @see #create(String, Class, String, String, String) for an explanation of the context.
     * @param editorContext the context
     */
    public void setContext(String editorContext) {
        if (editorContext != null) {
            setValue(ATTRIBUTE_CONTEXT, editorContext);
        }
    }

    /**
     * Configures the property category. In the install4j GUI, properties are grouped by their categories. If
     * no category has been specified, the default "Configuration" category is used.
     * <p>You can collapse certain categories by default with the
     * {@link Install4JBeanInfo#setCollapsedPropertyCategories(String[])} method.
     * @param propertyCategory the category
     */
    public void setPropertyCategory(String propertyCategory) {
        if (propertyCategory != null) {
            setValue(ATTRIBUTE_PROPERTY_CATEGORY, propertyCategory);
        }
    }

    public void setDisplayName(String displayName) {
        super.setDisplayName(displayName);
        if (displayName == null || displayName.trim().length() == 0) {
            setHidden(true);
        }
    }

    /**
     * Set an integer key by which properties are sorted. If no sort keys are specified, the properties will be sorted
     * in the order they were added to the bean info.
     * @param sortKey the sort key
     * @see Install4JBeanInfo#addPropertyDescriptor(Install4JPropertyDescriptor)
     */
    public void setSortKey(Integer sortKey) {
        if (sortKey != null) {
            setValue(ATTRIBUTE_SORT_KEY, sortKey);
        }
    }

    /**
     * Set a property change listener that is invoked when the property is changed in the IDE. The source of
     * the change event is the bean itself.
     * @param listener the listener
     */
    public void setPropertyChangeListener(PropertyChangeListener listener) {
        if (listener != null) {
            setValue(ATTRIBUTE_PROPERTY_CHANGE_LISTENER, listener);
        }
    }

    /**
     * Set the parent property under which this property should be shown in the install4j IDE.
     * The property sheet is a tree table and each property can have child nodes. By assigning parent
     * properties, you can build up a tree-like structure in the property sheet.
     * <p>
     * For boolean properties, only descendant properties are only shown if the boolean value of the property
     * is selected as <tt>true</tt>. If the user selection is false, all descendant properties will be hidden.
     * If the selection is changed to true, all descendant properties will be shown.
     * </p>
     * @param parentProperty the name of the parent property
     */
    public void setParentProperty(String parentProperty) {
        if (parentProperty != null) {
            setValue(ATTRIBUTE_PARENT_PROPERTY, parentProperty);
        }
    }

    /**
     * Sets a visibility discriminator for this property. This only makes sense if
     * {@link #setParentProperty(String)} has been called for this property descriptor.
     * Note that for a boolean parent properties, a default visibility discriminator is
     * set that hides children if the user selection of the parent property is <tt>false<</tt>.
     * @param visibilityDiscriminator the visibility discriminator
     * @see com.install4j.api.beaninfo.VisibilityDiscriminator
     */
    public void setVisibilityDiscriminator(VisibilityDiscriminator visibilityDiscriminator) {
        if (visibilityDiscriminator != null) {
            setValue(ATTRIBUTE_VISIBILITY_DISCRIMINATOR, visibilityDiscriminator);
        }
    }
}
