package com.install4j.api.launcher;

import com.install4j.runtime.installer.helper.apiimpl.ApplicationLauncherImpl;

import java.io.IOException;

/**
 * This class allows you to launch an installer application, such as an updater, from your own application.
 * It includes a mechanism for the installer application to shut down the invoking application with a
 * "Shutdown calling launcher" action.
 *
 * <p>In order to use this class, please include <tt>resource/i4jruntime.jar</tt> from your
 * install4j installation into your class path. You do not need to distribute this file
 * along with your application, install4j will do this automatically for you.
 * </p>
 *
 * @author ej-technologies GmbH
 *
 */
public class ApplicationLauncher {

    /**
     * Launch an installer application that you have defined in the install4j IDE.
     * @param applicationId the ID of the installer application to be launched. You can show the IDs of all installer applications on the
     * "Installer->Screens & actions" tab with the "Show IDs" tool bar button.
     * @param arguments the arguments you want to pass to the installer application. Can be <tt>null</tt> if no arguments
     * should be passed. Arguments like <tt>-Dkey=value</tt> will be passed as JVM parameters, just like when calling the installer
     * application from the command line.
     * @param blocking if this call should block until the installer application exits.
     * @param callback an optional call back for receiving notifications when the installer application exits or
     * when the installer application shuts down this JVM. Can be <tt>null</tt>
     * @throws IOException if there was an error starting the installer application
     */
    public static void launchApplication(String applicationId, String[] arguments, boolean blocking, Callback callback) throws IOException {
        ApplicationLauncherImpl.launchApplication(applicationId, arguments, blocking, callback);
    }

    /**
     * You can implement this interface to receive notifications when the installer application exits or needs to
     * shut down your application.
     * @see ApplicationLauncher#launchApplication(String, String[], boolean, com.install4j.api.launcher.ApplicationLauncher.Callback)
     */
    public interface Callback {
        /**
         * Invoked when the installer application has exited.
         * @param exitValue the exit value of installer application
         */
        public void exited(int exitValue);

        /**
         * Invoked before the installer appliaction shuts down this JVM.
         * This happens when a "Shutdown calling launcher" action is executed in the
         * installer application.
         */
        public void prepareShutdown();
    }

}
