package com.install4j.api.update;

import java.net.URL;

/**
 * Contains information about a single media file available for update. Instances of this
 * class are returned by an {@link UpdateDescriptor}.
 *
 * @author ej-technologies GmbH
 */
public interface UpdateDescriptorEntry {
    /**
     * Returns the media file ID for which this update is suitable.
     * <p>
     * Note: install4j creates one update descriptor entry for
     * each media file. If you discontinue a media file, you can duplicate a suitable update descriptor entry in
     * <tt>updates.xml</tt> and set its target media file ID to that of the discontinued media file.
     * @return the media file ID
     */
    String getTargetMediaFileId();

    /**
     * Returns the minimum version that is eligible for an update.
     * <p>
     * By default, this is empty string. If you want to set a minimum version, you can
     * change the <tt>updatableVersionMin</tt> attribute on the corresponding <tt>entry</tt> element
     * in the <tt>updates.xml</tt> file.
     * @return the minimum version
     */
    String getUpdatableVersionMin();

    /**
     * Returns the maximum version that is eligible for an update.
     * <p>
     * By default, this is empty string. If you want to set a maximum version, you can
     * change the <tt>updatableVersionMax</tt> attribute on the corresponding <tt>entry</tt> element
     * in the <tt>updates.xml</tt> file.
     * @return the maximum version
     */
    String getUpdatableVersionMax();

    /**
     * Returns the file name of the update installer.
     * @return the file name
     */
    String getFileName();

    /**
     * Returns the full URL from which the update installer can be downloaded.
     * This includes the file name and is based on the {@link UpdateDescriptor#getBaseUrl() base url}
     * of the {@link UpdateDescriptor update descriptor}.
     * @return the URL
     */
    URL getURL();

    /**
     * Returns the version of the available update.
     * @return the new version
     */
    String getNewVersion();

    /**
     * Returns the media file ID of the update installer that is represented by this entry.
     * @return the media file ID
     */
    String getNewMediaFileId();

    /**
     * Returns whether the media file is an installer or an archive.
     * @return <tt>true</tt> for an archive
     */
    boolean isArchive();

    /**
     * Returns the file size of the update installer in bytes. The size is determined at compile time.
     * @return the file size in bytes
     */
    long getFileSize();

    /**
     * Returns the file size of the update installer as a verbose string. The size is determined at compile time.
     * @return the file size as a string
     */
    String getFileSizeVerbose();

    /** Returns the identifier of the bundled JRE. If no JRE is bundled, this is the empty string.
     * Otherwise, this is the file name of the JRE bundle without the <tt>.tar.gz</tt> extension.
     * @return the ID of the bundled JRE or the empty string
     */
    String getBundledJre();

    /**
     * Returns the comment for this entry. By default, the comment is empty, you can add
     * comments in the <tt>comment</tt> elements in the <tt>updates.xml</tt> file. The default standalone
     * updater offers the user a hyperlink to display this comment for an available update.
     * @return the comment
     */
    String getComment();

    /**
     * Returns an additional attribute that you have added to the <tt>entry</tt> element in the <tt>updates.xml</tt> file
     * represented by this instance.
     * @param attributeName the name of the additional attribute
     * @param defaultValue the default value to be returned if the attribute does not exist
     * @return the attribute value
     */
    String getAdditionalAttribute(String attributeName, String defaultValue);
}
